% This script contains the code necessary for running the linear example 
% in Section 10.1 of the paper
%
% Thomas B. Schn, Adrian Wills and Brett Ninness. System Identification 
% of Nonlinear State-Space Models. Automatica, 47(1):39-49, January 2011. 
%

addpath('../UNIT');  % Add path to UNIT toolbox, this toolbox is free and 
                     % can be downloaded from http://sigpromu.org/idtoolbox/

opt.miter = 30;      % Max number of EM iterations
opt.LLdec = 1e-6;    % Min decrease of log-likelihood
theta0    = 0.3;     % Initial guess for the a parameter
runOldPS  = 0;       % Should the old O(N^2) particle smother be used
runNewPS  = 1;       % Should the new O(N) particle smother be used

%==========================================================
%===   Simulate the one dimensional state-space model   ===
%==========================================================
N = 100;          % Number of data
m.ss.A = 0.9;     m.ss.B = 0;
m.ss.C = 0.5;     m.ss.D = 0;
m.ss.Q = 0.1;     m.ss.S = 0;
m.ss.R = 0.1;
m.ss.X1 = 0;      m.ss.P1 = 0;  % Initial state (fully known)
x = zeros(1,N+1); y = zeros(1,N);
x(1) = m.ss.X1;
v = sqrt(m.ss.Q)*randn(1,N);    % Process noise sequence
e = sqrt(m.ss.R)*randn(1,N);    % Measurement noise sequence
for t=1:N
  x(t+1) = m.ss.A*x(t) + v(t);
  y(t)   = m.ss.C*x(t) + e(t);
end
z.y = y;

%===========================================
%===   EM algorithm using RTS smoother   ===
%===========================================
mEst = m;
mEst.ss.A = theta0;     % Initial parameter guess
for k = 1:opt.miter
  % E step
  g     = ks(z,mEst,opt);     % Smoother
  phi   = 0;   psi = 0;   sigma = 0;
  for t = 1:N
    phi   = phi + g.ss.xs(t+1)*g.ss.xs(t+1) + g.ss.Ps(1,1,t+1)^2;
    sigma = sigma + g.ss.xs(t).^2 + g.ss.Ps(1,1,t)^2;
    psi   = psi + g.ss.xs(t)*g.ss.xs(t+1) + g.ss.Ms(1,1,t);
  end;
  % M step
  a(k)      = psi/sigma;     % Store result
  mEst.ss.A = a(k);          % Update model
end;
aRTS = [theta0 a];

%===========================================================
%===   EM algorithm using a particle smoother (O(N^2))   ===
%===========================================================
if runOldPS
  M = 100;                    % Number of particles
  mEstPSDGA       = m;
  mEstPSDGA.ss.A  = theta0;   % Initial parameter guess
  mEstPSDGA.ss.P1 = 1;        % Spreas the initial particles a bit
  for k = 1:opt.miter
    % E step
    gPF    = pf(mEstPSDGA,M,z);         % Particle filter
    gPSDGA = psDGA(mEstPSDGA,M,z,gPF);  % Particle smoother
    gamma  = 0;
    for t=1:N-1
      gamma = gamma + sum(gPSDGA.ws(t+1,:).*gPSDGA.xPart(t+1,:).^2);
    end;
    psi = 0;
    for t = 1:N-1
      for i = 1:M
        psi = psi + sum(squeeze(gPSDGA.wij(t,i,:))'.*gPSDGA.xPart(t+1,:)*gPSDGA.xPart(t,i));
      end;
    end;
    % M step
    aPSDGA(k)      = psi/gamma;     % Store result
    mEstPSDGA.ss.A = aPSDGA(k);     % Update model
    disp(['Iteration nr: ' num2str(k) '  Current estimate of a: ' num2str(aPSDGA(k))])
  end;
  aPSDGA = [theta0 aPSDGA];
end;

%=========================================================
%===   EM algorithm using a particle smoother (O(N))   ===
%=========================================================
if runNewPS
  M = 100;                 % Number of particles
  mEstPS       = m;
  mEstPS.ss.A  = theta0;   % Initial parameter guess
  mEstPS.ss.P1 = 1;        % Spreas the initial particles a bit
  for k = 1:opt.miter
    % E step
    gPF    = pf(mEstPS,M,z);      % Particle filter
    gPS    = ps(mEstPS,M,gPF);    % Particle smoother
    gamma  = 0;
    for t=1:N
      gamma = gamma + (1/M)*sum(gPS.xPart(t,:).^2);
    end;
    psi = 0;
    for t = 1:N-1
      psi = psi + (1/M)*sum(gPS.xPart(t,:).*gPS.xPart(t+1,:));
    end;
    % M step
    aPS(k)      = psi/gamma;     % Store result
    mEstPS.ss.A = aPS(k);        % Update model
    disp(['Iteration nr: ' num2str(k) '  Current estimate of a: ' num2str(aPS(k))])
  end;
  aPS = [theta0 aPS];
end;

if runOldPS
  figure(1)
  plot(aRTS,'k')
  hold on;
  plot(aPSDGA,'r')
  hold off;
  xlabel('Iteration')
  ylabel('$\widehat{a}$','interpreter','latex','fontsize',14)
  legend('Optimal estimate','Using old particle smoother','Location','SouthEast')
  axis([0 30 0 1])  
end;

if runNewPS
  figure(2)
  plot(aRTS,'k')
  hold on;
  plot(aPS,'r')
  hold off;
  xlabel('Iteration')
  ylabel('$\widehat{a}$','interpreter','latex','fontsize',14)
  legend('Optimal estimate','Using new particle smoother','Location','SouthEast')
  axis([0 30 0 1])
end;
