% This script contains the code necessary for running the nonlinear example 
% in Section 10.2 of the paper
%
% Thomas B. Schn, Adrian Wills and Brett Ninness. System Identification 
% of Nonlinear State-Space Models. Automatica, 47(1):39-49, January 2011. 
%

clear;
opt.miter = 1000;     % Number of EM iterations

% Initialize randomly within 50% of the true value for all parameters
a0 = 0.5  + (0.5*0.5)*(-1 + 2*rand(1));
b0 = 25   + (0.5*25)*(-1 + 2*rand(1));
c0 = 8;     % not globally identifiable
d0 = 0.05 + (0.5*0.05)*(-1 + 2*rand(1));
Q0 = 0.05 + (0.5*0.05)*rand(1);
R0 = 0.1  + (0.5*0.1)*rand(1);
theta0 = [a0 b0 c0 d0 sqrt(Q0) sqrt(R0)];    % Initial guess for the parameters
% Note that we are estimating the square root factor of the covariance in
% order to enforce the fact that the covariance should be positive
% semi-definite.
disp(['Initial parameter guess, a: ' num2str(a0),'  b: ',num2str(b0),'  c: ',num2str(c0),'  d: ',num2str(d0),'  Q: ',num2str(Q0),'  R: ',num2str(R0)])

%====================================================
%===   Simulate the nonlinear state-space model   ===
%====================================================
N    = 100;            % Number of data
m.a  = 0.5;     m.b  = 25;
m.c  = 8;       m.d  = 0.05;
m.Q  = 0.05;    m.R  = 0.1;
m.X1 = 0;       m.P1 = 0;  % Initial state (fully known)
x = zeros(1,N+1); y = zeros(1,N);
x(1) = m.X1;
v = sqrt(m.Q)*randn(1,N);    % Process noise sequence
e = sqrt(m.R)*randn(1,N);    % Measurement noise sequence
for t=1:N
  x(t+1) = m.a*x(t) + m.b*x(t)/(1+x(t)^2) + m.c*cos(1.2*t) + v(t);
  y(t)   = m.d*x(t)^2 + e(t);
end
z.y     = y;
z.xTrue = x(1:N);

%==================================================
%===   EM algorithm using a particle smoother   ===
%==================================================
M       = 200;             % Number of particles
mEst    = m;
mEst.a  = theta0(1);   mEst.b  = theta0(2);   mEst.c  = theta0(3);
mEst.d  = theta0(4);   mEst.Q  = theta0(5)^2; mEst.R  = theta0(6)^2;
mEst.X1 = m.X1;            % Initial mean values known
mEst.P1 = 1;               % Spread the initial particles a bit
theta   = theta0;
mEstStore(1) = mEst;       % Store the interates
%options      = optimset('Display','off','LargeScale','off');
options      = optimset('LargeScale','off', 'Display','off','TolFun',1e-5,'TolX',1e-5); 
for k = 1:opt.miter
  % E step
  gPF = pf(mEst,M,z);      % Particle filter
  gPS = ps(mEst,M,gPF);    % Particle smoother

  % M step
  [theta,Qval] = fminunc('nl1NoGrad',theta,options,gPS.xPart,z.y);
  mEst.a = theta(1);   mEst.b = theta(2);
  mEst.c = m.c;           % Again, since it is not globally identifiable
  mEst.d = theta(4);   mEst.Q = theta(5)^2; mEst.R = theta(6)^2;
  mEstStore(k) = mEst;    % Store the iterates

  disp(['Iteration nr: ' num2str(k) ' Estimates, a: ' num2str(mEst.a),'  b: ',num2str(mEst.b),'  c: ',num2str(mEst.c),'  d: ',num2str(mEst.d),'  Q: ',num2str(mEst.Q),'  R: ',num2str(mEst.R)])
end;
